// This file is autogenerated. Changes will be overwritten.
#include "Z4VSQ6WDPV/moc_dbasefilewatcher.cpp"
#include "Z4VSQ6WDPV/moc_dcapfile.cpp"
#include "Z4VSQ6WDPV/moc_dcapmanager.cpp"
#include "Z4VSQ6WDPV/moc_dfilewatchermanager.cpp"
#include "XPH3BW4U2C/moc_dconfig.cpp"
#include "XPH3BW4U2C/moc_ddesktopentry.cpp"
#include "XPH3BW4U2C/moc_dsysinfo.cpp"
#include "BWZY4CD3W4/moc_dsettingsdconfigbackend.cpp"
#include "BWZY4CD3W4/moc_qsettingbackend.cpp"
#include "AQJCAY6EJ7/moc_dsettings.cpp"
#include "AQJCAY6EJ7/moc_dsettingsbackend.cpp"
#include "AQJCAY6EJ7/moc_dsettingsgroup.cpp"
#include "AQJCAY6EJ7/moc_dsettingsoption.cpp"
#include "HMJSVG7GSJ/moc_ddbusextendedabstractinterface.cpp"
#include "HMJSVG7GSJ/moc_ddbusinterface.cpp"
#include "HMJSVG7GSJ/moc_dexportedinterface.cpp"
#include "3DINGQWIAT/moc_dconfig_org_deepin_dtk_preference.cpp"
#include "KH43KSYMFX/moc_ddbusextendedpendingcallwatcher_p.cpp"
#include "KH43KSYMFX/moc_ddbusinterface_p.cpp"
