/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.ConfigManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef CONFIGMANAGER_INTERFACE_H
#define CONFIGMANAGER_INTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.desktopspec.ConfigManager
 */
class DSGConfig: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.desktopspec.ConfigManager"; }

public:
    DSGConfig(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~DSGConfig();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QDBusObjectPath> acquireManager(const QString &appid, const QString &name, const QString &subpath)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(appid) << QVariant::fromValue(name) << QVariant::fromValue(subpath);
        return asyncCallWithArgumentList(QStringLiteral("acquireManager"), argumentList);
    }

    inline QDBusPendingReply<> sync(const QString &path)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(path);
        return asyncCallWithArgumentList(QStringLiteral("sync"), argumentList);
    }

    inline QDBusPendingReply<> update(const QString &path)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(path);
        return asyncCallWithArgumentList(QStringLiteral("update"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

#endif
