/****************************************************************************
** Meta object code from reading C++ file 'ddbusextendedabstractinterface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/util/ddbusextendedabstractinterface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ddbusextendedabstractinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Core__DDBusExtendedAbstractInterface_t {
    QByteArrayData data[31];
    char stringdata0[528];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Core__DDBusExtendedAbstractInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Core__DDBusExtendedAbstractInterface_t qt_meta_stringdata_Dtk__Core__DDBusExtendedAbstractInterface = {
    {
QT_MOC_LITERAL(0, 0, 41), // "Dtk::Core::DDBusExtendedAbstr..."
QT_MOC_LITERAL(1, 42, 19), // "serviceValidChanged"
QT_MOC_LITERAL(2, 62, 0), // ""
QT_MOC_LITERAL(3, 63, 5), // "valid"
QT_MOC_LITERAL(4, 69, 20), // "serviceStartFinished"
QT_MOC_LITERAL(5, 90, 3), // "ret"
QT_MOC_LITERAL(6, 94, 15), // "propertyChanged"
QT_MOC_LITERAL(7, 110, 12), // "propertyName"
QT_MOC_LITERAL(8, 123, 5), // "value"
QT_MOC_LITERAL(9, 129, 19), // "propertyInvalidated"
QT_MOC_LITERAL(10, 149, 21), // "asyncPropertyFinished"
QT_MOC_LITERAL(11, 171, 24), // "asyncSetPropertyFinished"
QT_MOC_LITERAL(12, 196, 29), // "asyncGetAllPropertiesFinished"
QT_MOC_LITERAL(13, 226, 19), // "startServiceProcess"
QT_MOC_LITERAL(14, 246, 19), // "onPropertiesChanged"
QT_MOC_LITERAL(15, 266, 13), // "interfaceName"
QT_MOC_LITERAL(16, 280, 17), // "changedProperties"
QT_MOC_LITERAL(17, 298, 21), // "invalidatedProperties"
QT_MOC_LITERAL(18, 320, 22), // "onDBusNameOwnerChanged"
QT_MOC_LITERAL(19, 343, 4), // "name"
QT_MOC_LITERAL(20, 348, 8), // "oldOwner"
QT_MOC_LITERAL(21, 357, 8), // "newOwner"
QT_MOC_LITERAL(22, 366, 23), // "onAsyncPropertyFinished"
QT_MOC_LITERAL(23, 390, 24), // "QDBusPendingCallWatcher*"
QT_MOC_LITERAL(24, 415, 1), // "w"
QT_MOC_LITERAL(25, 417, 26), // "onAsyncSetPropertyFinished"
QT_MOC_LITERAL(26, 444, 31), // "onAsyncGetAllPropertiesFinished"
QT_MOC_LITERAL(27, 476, 7), // "watcher"
QT_MOC_LITERAL(28, 484, 29), // "onStartServiceProcessFinished"
QT_MOC_LITERAL(29, 514, 4), // "sync"
QT_MOC_LITERAL(30, 519, 8) // "useCache"

    },
    "Dtk::Core::DDBusExtendedAbstractInterface\0"
    "serviceValidChanged\0\0valid\0"
    "serviceStartFinished\0ret\0propertyChanged\0"
    "propertyName\0value\0propertyInvalidated\0"
    "asyncPropertyFinished\0asyncSetPropertyFinished\0"
    "asyncGetAllPropertiesFinished\0"
    "startServiceProcess\0onPropertiesChanged\0"
    "interfaceName\0changedProperties\0"
    "invalidatedProperties\0onDBusNameOwnerChanged\0"
    "name\0oldOwner\0newOwner\0onAsyncPropertyFinished\0"
    "QDBusPendingCallWatcher*\0w\0"
    "onAsyncSetPropertyFinished\0"
    "onAsyncGetAllPropertiesFinished\0watcher\0"
    "onStartServiceProcessFinished\0sync\0"
    "useCache"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Core__DDBusExtendedAbstractInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       2,  132, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x06 /* Public */,
       4,    1,   87,    2, 0x06 /* Public */,
       6,    2,   90,    2, 0x06 /* Public */,
       9,    1,   95,    2, 0x06 /* Public */,
      10,    1,   98,    2, 0x06 /* Public */,
      11,    1,  101,    2, 0x06 /* Public */,
      12,    0,  104,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    0,  105,    2, 0x0a /* Public */,
      14,    3,  106,    2, 0x08 /* Private */,
      18,    3,  113,    2, 0x08 /* Private */,
      22,    1,  120,    2, 0x08 /* Private */,
      25,    1,  123,    2, 0x08 /* Private */,
      26,    1,  126,    2, 0x08 /* Private */,
      28,    1,  129,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::UInt,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariant,    7,    8,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariantMap, QMetaType::QStringList,   15,   16,   17,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,   19,   20,   21,
    QMetaType::Void, 0x80000000 | 23,   24,
    QMetaType::Void, 0x80000000 | 23,   24,
    QMetaType::Void, 0x80000000 | 23,   27,
    QMetaType::Void, 0x80000000 | 23,   24,

 // properties: name, type, flags
      29, QMetaType::Bool, 0x00095103,
      30, QMetaType::Bool, 0x00095103,

       0        // eod
};

void Dtk::Core::DDBusExtendedAbstractInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DDBusExtendedAbstractInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->serviceValidChanged((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 1: _t->serviceStartFinished((*reinterpret_cast< const quint32(*)>(_a[1]))); break;
        case 2: _t->propertyChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 3: _t->propertyInvalidated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->asyncPropertyFinished((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->asyncSetPropertyFinished((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->asyncGetAllPropertiesFinished(); break;
        case 7: _t->startServiceProcess(); break;
        case 8: _t->onPropertiesChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3]))); break;
        case 9: _t->onDBusNameOwnerChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 10: _t->onAsyncPropertyFinished((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 11: _t->onAsyncSetPropertyFinished((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 12: _t->onAsyncGetAllPropertiesFinished((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 13: _t->onStartServiceProcessFinished((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DDBusExtendedAbstractInterface::*)(const bool ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDBusExtendedAbstractInterface::serviceValidChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DDBusExtendedAbstractInterface::*)(const quint32 ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDBusExtendedAbstractInterface::serviceStartFinished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DDBusExtendedAbstractInterface::*)(const QString & , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDBusExtendedAbstractInterface::propertyChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DDBusExtendedAbstractInterface::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDBusExtendedAbstractInterface::propertyInvalidated)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DDBusExtendedAbstractInterface::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDBusExtendedAbstractInterface::asyncPropertyFinished)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DDBusExtendedAbstractInterface::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDBusExtendedAbstractInterface::asyncSetPropertyFinished)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DDBusExtendedAbstractInterface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DDBusExtendedAbstractInterface::asyncGetAllPropertiesFinished)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DDBusExtendedAbstractInterface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->sync(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->useCache(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DDBusExtendedAbstractInterface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSync(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setUseCache(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Core::DDBusExtendedAbstractInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Core__DDBusExtendedAbstractInterface.data,
    qt_meta_data_Dtk__Core__DDBusExtendedAbstractInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Core::DDBusExtendedAbstractInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Core::DDBusExtendedAbstractInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Core__DDBusExtendedAbstractInterface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int Dtk::Core::DDBusExtendedAbstractInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Dtk::Core::DDBusExtendedAbstractInterface::serviceValidChanged(const bool _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Dtk::Core::DDBusExtendedAbstractInterface *>(this), &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Dtk::Core::DDBusExtendedAbstractInterface::serviceStartFinished(const quint32 _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Dtk::Core::DDBusExtendedAbstractInterface *>(this), &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Dtk::Core::DDBusExtendedAbstractInterface::propertyChanged(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Dtk::Core::DDBusExtendedAbstractInterface::propertyInvalidated(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Dtk::Core::DDBusExtendedAbstractInterface::asyncPropertyFinished(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Dtk::Core::DDBusExtendedAbstractInterface::asyncSetPropertyFinished(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Dtk::Core::DDBusExtendedAbstractInterface::asyncGetAllPropertiesFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
