/****************************************************************************
** Meta object code from reading C++ file 'dfilesystemwatcher.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/filesystem/dfilesystemwatcher.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfilesystemwatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Core__DFileSystemWatcher_t {
    QByteArrayData data[15];
    char stringdata0[171];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Core__DFileSystemWatcher_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Core__DFileSystemWatcher_t qt_meta_stringdata_Dtk__Core__DFileSystemWatcher = {
    {
QT_MOC_LITERAL(0, 0, 29), // "Dtk::Core::DFileSystemWatcher"
QT_MOC_LITERAL(1, 30, 11), // "fileDeleted"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 4), // "path"
QT_MOC_LITERAL(4, 48, 4), // "name"
QT_MOC_LITERAL(5, 53, 20), // "fileAttributeChanged"
QT_MOC_LITERAL(6, 74, 10), // "fileClosed"
QT_MOC_LITERAL(7, 85, 9), // "fileMoved"
QT_MOC_LITERAL(8, 95, 8), // "fromPath"
QT_MOC_LITERAL(9, 104, 8), // "fromName"
QT_MOC_LITERAL(10, 113, 6), // "toPath"
QT_MOC_LITERAL(11, 120, 6), // "toName"
QT_MOC_LITERAL(12, 127, 11), // "fileCreated"
QT_MOC_LITERAL(13, 139, 12), // "fileModified"
QT_MOC_LITERAL(14, 152, 18) // "_q_readFromInotify"

    },
    "Dtk::Core::DFileSystemWatcher\0fileDeleted\0"
    "\0path\0name\0fileAttributeChanged\0"
    "fileClosed\0fileMoved\0fromPath\0fromName\0"
    "toPath\0toName\0fileCreated\0fileModified\0"
    "_q_readFromInotify"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Core__DFileSystemWatcher[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   49,    2, 0x06 /* Public */,
       5,    2,   54,    2, 0x06 /* Public */,
       6,    2,   59,    2, 0x06 /* Public */,
       7,    4,   64,    2, 0x06 /* Public */,
      12,    2,   73,    2, 0x06 /* Public */,
      13,    2,   78,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    0,   83,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,    8,    9,   10,   11,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    4,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void Dtk::Core::DFileSystemWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DFileSystemWatcher *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fileDeleted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])), QPrivateSignal()); break;
        case 1: _t->fileAttributeChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])), QPrivateSignal()); break;
        case 2: _t->fileClosed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])), QPrivateSignal()); break;
        case 3: _t->fileMoved((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])), QPrivateSignal()); break;
        case 4: _t->fileCreated((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])), QPrivateSignal()); break;
        case 5: _t->fileModified((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])), QPrivateSignal()); break;
        case 6: _t->d_func()->_q_readFromInotify(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal);
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DFileSystemWatcher::fileDeleted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal);
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DFileSystemWatcher::fileAttributeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal);
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DFileSystemWatcher::fileClosed)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DFileSystemWatcher::*)(const QString & , const QString & , const QString & , const QString & , QPrivateSignal);
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DFileSystemWatcher::fileMoved)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal);
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DFileSystemWatcher::fileCreated)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal);
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DFileSystemWatcher::fileModified)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Dtk::Core::DFileSystemWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Dtk__Core__DFileSystemWatcher.data,
    qt_meta_data_Dtk__Core__DFileSystemWatcher,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Dtk::Core::DFileSystemWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Core::DFileSystemWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Dtk__Core__DFileSystemWatcher.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DObject"))
        return static_cast< DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Core::DFileSystemWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Core::DFileSystemWatcher::fileDeleted(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Dtk::Core::DFileSystemWatcher::fileAttributeChanged(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Dtk::Core::DFileSystemWatcher::fileClosed(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Dtk::Core::DFileSystemWatcher::fileMoved(const QString & _t1, const QString & _t2, const QString & _t3, const QString & _t4, QPrivateSignal _t5)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Dtk::Core::DFileSystemWatcher::fileCreated(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Dtk::Core::DFileSystemWatcher::fileModified(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
