/****************************************************************************
** Meta object code from reading C++ file 'dfilesystemwatcher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/filesystem/dfilesystemwatcher.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfilesystemwatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk4Core18DFileSystemWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Core::DFileSystemWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk4Core18DFileSystemWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Core::DFileSystemWatcher",
        "fileDeleted",
        "",
        "path",
        "name",
        "fileAttributeChanged",
        "fileClosed",
        "fileMoved",
        "fromPath",
        "fromName",
        "toPath",
        "toName",
        "fileCreated",
        "fileModified",
        "_q_readFromInotify"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fileDeleted'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'fileAttributeChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'fileClosed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'fileMoved'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'fileCreated'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'fileModified'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot '_q_readFromInotify'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DFileSystemWatcher, qt_meta_tag_ZN3Dtk4Core18DFileSystemWatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Core::DFileSystemWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk4Core18DFileSystemWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk4Core18DFileSystemWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk4Core18DFileSystemWatcherE_t>.metaTypes,
    nullptr
} };

void Dtk::Core::DFileSystemWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DFileSystemWatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fileDeleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])), QPrivateSignal()); break;
        case 1: _t->fileAttributeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])), QPrivateSignal()); break;
        case 2: _t->fileClosed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])), QPrivateSignal()); break;
        case 3: _t->fileMoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])), QPrivateSignal()); break;
        case 4: _t->fileCreated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])), QPrivateSignal()); break;
        case 5: _t->fileModified((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])), QPrivateSignal()); break;
        case 6: _t->d_func()->_q_readFromInotify(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal)>(_a, &DFileSystemWatcher::fileDeleted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal)>(_a, &DFileSystemWatcher::fileAttributeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal)>(_a, &DFileSystemWatcher::fileClosed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFileSystemWatcher::*)(const QString & , const QString & , const QString & , const QString & , QPrivateSignal)>(_a, &DFileSystemWatcher::fileMoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal)>(_a, &DFileSystemWatcher::fileCreated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFileSystemWatcher::*)(const QString & , const QString & , QPrivateSignal)>(_a, &DFileSystemWatcher::fileModified, 5))
            return;
    }
}

const QMetaObject *Dtk::Core::DFileSystemWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Core::DFileSystemWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk4Core18DFileSystemWatcherE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DObject"))
        return static_cast< DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Dtk::Core::DFileSystemWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Dtk::Core::DFileSystemWatcher::fileDeleted(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void Dtk::Core::DFileSystemWatcher::fileAttributeChanged(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void Dtk::Core::DFileSystemWatcher::fileClosed(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void Dtk::Core::DFileSystemWatcher::fileMoved(const QString & _t1, const QString & _t2, const QString & _t3, const QString & _t4, QPrivateSignal _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 4
void Dtk::Core::DFileSystemWatcher::fileCreated(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 5
void Dtk::Core::DFileSystemWatcher::fileModified(const QString & _t1, const QString & _t2, QPrivateSignal _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
